/**********************************************************************************
 Persistence of Vision Ray Tracer Scene Description File
 File name   : KrystalShaper_apophyllite.pov
 Version     : 3.8
 Created By  : KrystalShaper, JCrystalSoft (C)2021, www.jcrystal.com
               KrystalShaper   2018 JCrystalSoft

 Description : Fluorapophyllite-(K) crystal
               https://en.wikipedia.org/wiki/Apophyllite
               comments on the material by Alain Martel at:
               http://news.povray.org/povray.binaries.images/thread/%3Cweb.6147c32b226a4256cb705ca46e741498%40news.povray.org%3E/
                              
 Date        : October 2021
 Author      : Thomas de Groot
 
 Copyright (C) 2021. All rights reserved.
 This file is licensed under the terms of the CC-LGPL.
 Use and/or modification of this scene is free, except for commercial purposes.
 Commercial distribution is not allowed without written permission from the author.
**********************************************************************************/

//+w640 +h640 +a0.3 +am2 +bm2 +bs8 +wt7

//choose the following if you want to use stochastic rendering:
//+w640 +h640 +am3 +a0.1 +ac0.90 +r3 +wt7

#version 3.8;

#include "rand.inc"
#include "strings.inc"

//#declare Axes       = off;
#declare Stochastic = off;            //if you do a stochastic render, this influences 'count' in global_settings
#declare Photons    = off;            //when 'on', SavePhot can be activated
#declare SavePhot   = on;             //'off' means reading the saved photons
#declare Name       = "apophyllite_rand01";  //two files available: "apophyllite", or: "apophyllite_rand01"
#declare Geometry   = 2;              //0=edges; 1=polygons; 2=mesh2
#declare CrystalRot = 18;             //rotation around the y-axis
#declare R          = seed(320439);

//[Global]
global_settings { 
  assumed_gamma 1.0 
  max_trace_level 5
  radiosity {
    pretrace_start 0.08           // start pretrace at this size
    pretrace_end   0.04          // end pretrace at this size
    //pretrace_start 16/max(image_width,image_height)
    //pretrace_end    4/max(image_width,image_height) 
    #if(Stochastic)
      count 10
    #else
      count 50, 1000              // higher -> higher quality (1..1600) [35]
    #end
    nearest_count 10, 5           // higher -> higher quality (1..10) [5]
    error_bound 1                 // higher -> smoother, less accurate [1.8]
    recursion_limit 2             // how much interreflections are calculated (1..5+) [2]
    low_error_factor .3           // reduce error_bound during last pretrace step
    gray_threshold 0.0            // increase for weakening colors (0..1) [0]
    minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
    maximum_reuse 0.1							// new parameter 3.7 [0.2]
    brightness 1                  // brightness of radiosity effects (0..1) [1]

    adc_bailout 0.01/2
    normal off                   // take surface normals into account [off]
    media off                    // take media into account [off]
    always_sample off           // turn sampling in final trace off [on]
    //max_sample 1.0              // maximum brightness of samples
  }
  #if (Photons)
  photons {
    spacing 0.01                 // specify the density of photons
    //count 100000              // alternatively use a total number of photons
    autostop 0
    jitter .4
    media 100
    #if (SavePhot)
      save_file "Apophyllite.ph"
    #else
      load_file "Apophyllite.ph"
    #end
  }
  #end
}

//==================================================================================================
// [Fluid Inclusions]
#declare AirMat =
material {
	texture {
	  pigment {rgbt <0.96, 0.96, 0.96, 0.40> }
	  finish {diffuse albedo 0.8 specular albedo 0.2 roughness 0.001}
	}
	interior {ior 1.0}
}
	
#local MinBox0 = <-1.5, -1.5, -1.5>;
#local MaxBox0 = <1.5, 1.5, 1.5>;
//scaling the bounding box to your needs:
#local Scale  = <0.75, 0.75, 0.75>;
#local MinBox = MinBox0*Scale;
#local MaxBox = MaxBox0*Scale;

//---------------------------------------------------------------------
//just to control how the bounding relates to the crystal
#local SphereBound =
sphere {
  0, MaxBox0.y 
  scale Scale
  pigment {rgb <0.9, 0.5, 0.5> transmit 0.8}
  no_shadow
  hollow
}

//if you want to see this bounding sphere, uncomment the following line
//SphereBound
//---------------------------------------------------------------------

#include "Sphere_POV_geom.inc"
#local Sphere = object {Sphere_ scale 0.0025}

//Convert bounding box into ranges
#local Xrange = MaxBox.x - MinBox.x;
#local Yrange = MaxBox.y - MinBox.y;
#local Zrange = MaxBox.z - MinBox.z;

#local Gauss = on;
#local Number = 5000;

#declare Inclusions =
union {
  #for (N, 0, Number)
    #if (Gauss)
      //Rand_Gauss(Mu,Sigma,Rand)
      //Mu = Mean : variation shifts the position along the corresponding axis 
      //Sigma = Standard deviation : distribution variation along the corresponding axis
	    #local PosX = Rand_Gauss(0.52, 0.05, R)*Xrange + MinBox.x; //(0.50, 075)
	    #local PosY = Rand_Gauss(0.50, 0.15, R)*Yrange + MinBox.y; //(0.50, 015)
	    #local PosZ = Rand_Gauss(0.50, 0.05, R)*Zrange + MinBox.z; //(0.50, 075)
	  #else
	    #local PosX = Rand_Cauchy(0.5, 0.15, R)*Xrange-MaxBox.y;
	    #local PosY = Rand_Cauchy(0.5, 0.15, R)*Yrange-MaxBox.y;
	    #local PosZ = Rand_Cauchy(0.5, 0.15, R)*Zrange-MaxBox.y;
	  #end
    #local Pos  = <PosX, PosY, PosZ>;
    object {Sphere scale RRand(0.9, 1.1, R) translate Pos}
  #end
  hollow
  material {AirMat}
  #if (Photons)
    photons {  // photon block for an object
      target 1.0
      refraction on
      reflection on
    }
  #end
}

//==================================================================================================
#if (Geometry = 0)
  //[Edge Cylinders]
  #include concat (Name, ".inc")
  object {Apophyllite_cylinders
    texture {
      pigment {srgb <0.9, 0.7, 0.7>}
      finish {
        diffuse 0.6
        specular 0.2
        roughness 0.001
      }
      normal {granite 1 scale 0.001}
    }
    rotate CrystalRot*y
  } //union

#elseif (Geometry = 1)
  //[Polygons]
  #include concat (Name, ".inc")
  #declare Apophyllite =
  object {Apophyllite_planes}

#else //Geometry=2
  //[mesh2]
  #include concat (Name, "_POV_geom.inc")
  #declare Apophyllite = object {apophyllite_ /*hollow*/}

#end  //of Geometry choice

//----------------------------------------------------------------------Adding material
#if (Geometry = 1 | Geometry = 2) 
  #declare Apophyllite =
  object {Apophyllite
    #local Color = <0.00, 0.95, 0.70>; //Fluorapophyllite-(K)
    material {
      texture {
        //pigment {rgbt 1}
        pigment{
          granite scale 1
          color_map {
            [0 rgb 1 transmit 0.99]
            [1 rgb 1 transmit 0.85]
          }
          warp {turbulence 0.33}
        } //pigment
        finish {reflection {0, 1 fresnel} conserve_energy}
        normal {
          marble warp {turbulence 0.33}
          normal_map{
            //[0.0 crackle solid scale 0.1*y]
            [0.0 cells 0.2 scale 0.2 warp {turbulence 0.33}]
            [0.5 wrinkles 0]
            [1.0 granite 0.05 scale 0.1 warp {turbulence 0.1}]
          }
          rotate 360*rand(R)*y
          translate 100*rand(R)
        } //normal
      } //texture
      interior {
        fade_color srgb Color//desired colour
        fade_power 1001 //how the path length affect the colour
        fade_distance 1.9 //base distance to be used for the fading
        //Adjust fade_distance to change the intensity of the colour of the crystal.
        //If your colour get to saturated, that mean that your fade_distance is to short, so, just increase fade_distance as needed.
        //Also, to get realistic results, fade_power need to be 1001 or 1, not 2. 
    
        // apophyllite ior from classicgems.net
        ior 1.536

        // an unrealistic dispersion value
        dispersion 1.075 dispersion_samples 7

        media {
          intervals 1
          samples 10
          absorption srgb 1-Color  //Use the complement of your colour. That's 1 minus the desired colour.

          density {
            spherical warp {turbulence 0.33}
            density_map {
              [0.20 rgb <0, 0, 0>]
              [0.70 granite scale 0.01]
            }
            //scale <0.5, 1.0, 0.5>
          } //density
        } //media
      } //interior
    } //material
    #if (Photons)
      photons {  // photon block for an object
        target 1.0
        refraction on
        reflection on
      }
    #end
  } //end of Apophyllite object (either intersection of planes, or mesh2)

#end  //material for polygons or mesh2

//----------------------------------------------------------------------finalizing crystal object
#if (Geometry != 0)
  //making sure the inclusions are restricted to the crystal
  #declare Inclusions =
  intersection {
    object {Inclusions}
    object {Apophyllite}
  }

  //difference{} is extremely slow in comparison! But would it be necessary
  //for inclusion media? not sure if it is worthwhile...  
  union {
    object {Apophyllite}
    object {Inclusions}
    rotate CrystalRot*y
  } //union
#end

//==================================================================================================
//[Camera Position]
camera {
  location <-0.2, 5, -10.0> 
  look_at <0, 0, 0>
  right  x*image_width/image_height
  angle 20.0 
}

//[Light Sources]
light_source { 
  <60, 100, -50.0> 
  rgb <0.9, 0.9, 0.9>*1.9 
  spotlight
  point_at <0, 0, 0>
  radius 1.5
  falloff 2.5
  tightness 10  
  area_light <-2,0,0>, <0,-2,0>, 3, 3  adaptive 1 
  #if (Photons)
  photons {
    refraction on
    reflection on
  }
  #end
}

//[Scene Background]
sphere {
  0, 1000 hollow //or inverse  
  pigment { 
    image_map { exr "probe_combo_blurSmall.exr" map_type 1 interpolate 3 } rotate y*-90
  }
  finish {emission 0.5 diffuse 0}
  #if (Photons)
  photons {
    refraction on
    reflection on
  }
  #end
  //no_image
}

plane {
  y, -1.5 
  pigment {rgb 0.4} 
  finish {diffuse 0.8 specular 0.1 roughness 0.001 emission 0}
  normal {granite 0.3 scale 0.001}
}
/*
#if (Axes)
  #include "DrawTheAxes.inc"
  MyAxis(5.0, 5.0, 5.0, 0.01)
#end
*/
